--[[----------------------------------------------------------
	CLS Menu Settings
 ]]-----------------------------------------------------------

--- Returns HEX representation of num
local function num2hex(num)
	local hexstr = '0123456789abcdef'
	local s = ''
	while num > 0 do
		local mod = math.fmod(num, 16)
		mod = math.floor(mod)
		s = string.sub(hexstr, mod+1, mod+1) .. s
		num = math.floor(num / 16)
	end
	if s == '' then s = '00' end
	if (string.len(s) < 2) then
		s = '0' .. s
	end
	return s
end

local function GetColor(word)
	-- get the saved setting, of the format |cRRGGBB
	local colorString = CLog.settings[word .. "Color"] or CLog.Default[word .. "Color"] or "|cFFFFFF"
	-- burst it into substrings, translate the numbers
	local r = tonumber(string.sub(colorString, 3, 4), 16) / 100
	local g = tonumber(string.sub(colorString, 5, 6), 16) / 100
	local b = tonumber(string.sub(colorString, 7, 8), 16) / 100
	return r, g, b
end

local function SetColor(word, r, g, b)
	local colorString = "|c"
	local colorString = colorString .. num2hex(r * 255)
	local colorString = colorString .. num2hex(g * 255)
	local colorString = colorString .. num2hex(b * 255)
	CLog.settings[word .. "Color"] = colorString
end

function CLog.InitMenu()
	CLog.SettingsHidden = false

	local panelData = {
		type = "panel",
		name = "CLS",
		displayName = "Combat Log Statistics Settings",
		author = "Zeakfury & katkat42",
		version = "3.0.5",
		registerForRefresh = true,
		registerForDefaults = true,
		slashCommand = "/clssettings",
	}
	CLog.LAM:RegisterAddonPanel("CLS", panelData)

	local optionsTable = {
		[1] = {
			type = "header",
			name = "|cF0C300Display Options",
		},
		[2] = {
			type = "checkbox",
			name = "Hide Main Window",
			tooltip = "Toggle Show/Hide the Main Window",
			getFunc = function() return CLog.settings.HiddenM end,
			setFunc = function(val)
				CLog.settings.HiddenM = val
				CLog.MainWindow:SetHidden(val)
			end,
			default = CLog.Default.HiddenM,
		},
		[3] = {
			type = "checkbox",
			name = "Lock Main Window",
			tooltip = "Lock Main Window in place?",
			getFunc = function() return CLog.settings.LockM end,
			setFunc = function(val)
				CLog.settings.LockM = val
				CLog.MainWindow:SetMovable(not val)
			end,
			default = CLog.Default.LockM,
		},
		[4] = {
			type = "checkbox",
			name = "Hide DPS Window",
			tooltip = "Toggle Show/Hide the DPS Window",
			getFunc = function() return CLog.settings.HiddenD end,
			setFunc = function(val)
				CLog.settings.HiddenD = val
				CLog.DPSWindow:SetHidden(val)
			end,
			default = CLog.Default.HiddenD,
		},
		[5] = {
			type = "checkbox",
			name = "Lock DPS Window",
			tooltip = "Lock DPS Window in place?",
			getFunc = function() return CLog.settings.LockD end,
			setFunc = function(val)
				CLog.settings.LockD = val
				CLog.DPSWindow:SetMovable(not val)
			end,
			default = CLog.Default.LockD,
		},
		[6] = {
			type = "dropdown",
			name = "Display Combat in Chat",
			tooltip = "Show/Hide Combat Log Data in Chat Window",
			choices = {"None", "First Tab", "Own Tab", "Both"},
			getFunc = function() return CLog.settings.Chat end,
			setFunc = function(val)
				local old = CLog.settings.Chat
				CLog.settings.Chat = val 
				CLog.InitChatTab()
				-- it's stupid that we have to reload the UI when adding or removing a tab from
				-- the chat box -- I use the exact same functions from the command line and they
				-- appeared just fine. But not when I'm in the settings menu. Weird.
				if ((old == "None" or old == "First Tab") and (val == "Own Tab" or val == "Both"))
					or ((old == "Own Tab" or old == "Both") and (val == "None" or val == "First Tab")) then
					ReloadUI()
				end
			end,
			warning = "Can Reload UI",
			default = CLog.Default.Chat,
		},
		[7] = {
			type = "checkbox",
			name = "Display TimeStamp",
			tooltip = "Toggle the TimeStamp in the Combat Log",
			getFunc = function() return CLog.settings.Timestamp end,
			setFunc = function(val) CLog.settings.Timestamp = val end,
			default = CLog.Default.Timestamp,
		},
		[8] = {
			type = "dropdown",
			name = "Text Direction",
			tooltip = "Which Direction the text pops up in Main Window",
			choices = {"UP", "DOWN"},
			getFunc = function() return CLog.settings.Direction end,
			setFunc = function(direction)
				if CLog.settings.Direction ~= direction then
					CLog.settings.Direction = direction
					ReloadUI()
				end
			end,
			warning = "Reloads UI",
			default = CLog.Default.Direction,
		},
		[9] = {
			type = "slider",
			name = "Max Lines Shown",
			tooltip = "Maximum number of lines shown in the main window",
			min = 0,
			max = 30,
			step = 1,
			getFunc = function() return CLog.settings.MaxLines end,
			setFunc = function(value)
				CLog.settings.MaxLines = value
				ReloadUI()
			end,
			warning = "Reloads UI",
			default = CLog.Default.MaxLines,
		},
		[10] = {
			type = "slider",
			name = "Max Saved Lines",
			tooltip = "Maximum number of lines saved for scrolling (0 = unlimited)",
			min = 0,
			max = 1000,
			step = 1,
			getFunc = function() return CLog.settings.MaxLinesSaved end,
			setFunc = function(value) CLog.settings.MaxLinesSaved = value end,
			default = CLog.Default.MaxLinesSaved,
		},
		[11] = {
			type = "checkbox",
			name = "Fade CLS Main Window when not in Combat",
			tooltip = "Fade/Don't Fade CLS window when not in combat",
			getFunc = function() return CLog.settings.FadesWithChat end,
			setFunc = function(val) CLog.settings.FadesWithChat = val end,
			default = CLog.Default.FadesWithChat,
		},
		[12] = {
			type = "checkbox",
			name = "Fade Main Window Text when not in Combat",
			tooltip = "Permanently display the log's text, or fade it with its window?",
			getFunc = function() return CLog.settings.FadeTextToo end,
			setFunc = function(val) CLog.settings.FadeTextToo = val end,
			disabled = function() return not CLog.settings.FadesWithChat end,
			default = CLog.Default.FadeTextToo,
		},
		[13] = {
			type = "checkbox",
			name = "Hide DPS Meter when not in Combat",
			tooltip = "Hide/Don't Hide the DPS Meter window when not in combat",
			getFunc = function() return CLog.settings.FadeDPSWithChat end,
			setFunc = function(val) CLog.settings.FadeDPSWithChat = val end,
			default = CLog.Default.FadeDPSWithChat,
		},
		[14] = {
			type = "slider",
			name = "Fade Timer",
			tooltip = "How much time to stay visible after combat ends",
			min = 1,
			max = 60,
			step = 1,
			getFunc = function() return CLog.settings.FadeTimer end,
			setFunc = function(value) CLog.settings.FadeTimer = value end,
			disabled = function() return not CLog.settings.FadesWithChat and not CLog.settings.FadeDPSWithChat end,
			default = CLog.Default.FadeTimer,
		},
		[15] = {
			type = "checkbox",
			name = "User Buffer for DPS Update",
			tooltip = "Will Decrease Performance if Off",
			getFunc = function() return CLog.settings.Buffer end, 
			setFunc = function(val) CLog.settings.Buffer = val end,
			default = CLog.Default.Buffer,
		}, 
		[16] = {
			type = "slider",
			name = "Buffer Update",
			tooltip = "Sets how long to buffer to update DPS window in tenths of a second",
			min = 1,
			max = 30,
			step = 1,
			getFunc = function() return CLog.settings.BufferSpeed*10 end,
			setFunc = function(value) CLog.settings.BufferSpeed = value*0.1 end,
			disabled = function() return not CLog.settings.Buffer end,
			default = CLog.Default.BufferSpeed,
		},

		[17] = {
			type = "header",
			name = "|cF0C300Cosmetics|r",
		},
		[18] = {
			type = "slider",
			name = "Font Size",
			tooltip = "Move to Change Font Size",
			min = 10,
			max = 40,
			step = 1,
			getFunc = function() return CLog.settings.FontSize end,
			setFunc = function(value)
				CLog.settings.FontSize = value
				for i = 1, #CLog.textbox , 1 do 
					CLog.textbox[i]:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
					CLog.textbox[i]:SetDimensions(1000, CLog.settings.FontSize+1)
					CLog.textbutton[i]:SetDimensions(20, CLog.settings.FontSize+1)
					CLog.textbutton[i]:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
				end	
				CLog.DPSWindow.LabelTex:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
				CLog.DPSWindow.LabelVal:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
				CLog.AbilityWindow.label:SetFont(CLog.Font .. "|" .. CLog.settings.FontSize .. "|" .. CLog.FontStyle)
			end,
			default = CLog.Default.FontSize,
		},
		[19] = {
			type = "checkbox",
			name = "Toggle Name Colors",
			tooltip = "Toggle the colors of Names",
			getFunc = function() return CLog.settings.NameColor end, 
			setFunc = function(val) CLog.settings.NameColor = val end,
			default = CLog.Default.NameColor,
		},
		[20] = {
			type = "submenu",
			name = "Set Name Colors",
			tooltip = "Expand to set colors for the names of combat actors",
			controls = {
				[1] = {
					type = "colorpicker",
					name = "You",
					getFunc = function() return GetColor("You") end,
					setFunc = function(r, g, b) SetColor("You", r, g, b) end,
					default = CLog.Default.YouColor,
				},
				[2] = {
					type = "colorpicker",
					name = "Group",
					getFunc = function() return GetColor("Group") end,
					setFunc = function(r, g, b) SetColor("Group", r, g, b) end,
					default = CLog.Default.GroupColor,
				},
				[3] = {
					type = "colorpicker",
					name = "Ally",
					getFunc = function() return GetColor("Ally") end,
					setFunc = function(r, g, b) SetColor("Ally", r, g, b) end,
					default = CLog.Default.AllyColor,
				},
				[4] = {
					type = "colorpicker",
					name = "Enemy",
					getFunc = function() return GetColor("Enemy") end,
					setFunc = function(r, g, b) SetColor("Enemy", r, g, b) end,
					default = CLog.Default.EnemyColor,
				},
				[5] = {
					type = "colorpicker",
					name = "NPC",
					getFunc = function() return GetColor("NPC") end,
					setFunc = function(r, g, b) SetColor("NPC", r, g, b) end,
					default = CLog.Default.NPCColor,
				},
				[6] = {
					type = "colorpicker",
					name = "Pet",
					getFunc = function() return GetColor("Pet") end,
					setFunc = function(r, g, b) SetColor("Pet", r, g, b) end,
					default = CLog.Default.PetColor,
				},
			},
		},
		[21] = {
			type = "checkbox",
			name = "Toggle Ability Colors",
			tooltip = "Toggles the colors of Abilities",
			getFunc = function() return CLog.settings.AbilityColor end, 
			setFunc = function(val) CLog.settings.AbilityColor = val end,
			default = CLog.Default.AbilityColor,
		},
		[22] = {
			type = "submenu",
			name = "Set Ability Colors",
			tooltip = "Expand to set colors for the names of abilities by damage type",
			controls = {
				[1] = {
					type = "colorpicker",
					name = "Healing",
					getFunc = function() return GetColor("Heal") end,
					setFunc = function(r, g, b) SetColor("Heal", r, g, b) end,
					default = CLog.Default.HealColor,
				},
				[2] = {
					type = "colorpicker",
					name = "Generic",
					getFunc = function() return GetColor("Generic") end,
					setFunc = function(r, g, b) SetColor("Generic", r, g, b) end,
					default = CLog.Default.GenericColor,
				},
				[3] = {
					type = "colorpicker", 
					name = "Physical Damage",
					getFunc = function() return GetColor("Physical") end,
					setFunc = function(r, g, b) SetColor("Physical", r, g, b) end,
					default = CLog.Default.PhysicalColor,
				},
				[4] = {
					type = "colorpicker",
					name = "Fire Damage",
					getFunc = function() return GetColor("Fire") end,
					setFunc = function(r, g, b) SetColor("Fire", r, g, b) end,
					default = CLog.Default.FireColor,
				},
				[5] = {
					type = "colorpicker",
					name = "Shock Damage",
					getFunc = function() return GetColor("Shock") end,
					setFunc = function(r, g, b) SetColor("Shock", r, g, b) end,
					default = CLog.Default.ShockColor,
				},
				[6] = {
					type = "colorpicker",
					name = "Oblivion Damage",
					getFunc = function() return GetColor("Oblivion") end,
					setFunc = function(r, g, b) SetColor("Oblivion", r, g, b) end,
					default = CLog.Default.OblivionColor,
				},
				[7] = {
					type = "colorpicker",
					name = "Cold Damage",
					getFunc = function() return GetColor("Cold") end,
					setFunc = function(r, g, b) SetColor("Cold", r, g, b) end,
					default = CLog.Default.ColdColor,
				},
				[8] = {
					type = "colorpicker",
					name = "Earth Damage",
					getFunc = function() return GetColor("Earth") end,
					setFunc = function(r, g, b) SetColor("Earth", r, g, b) end,
					default = CLog.Default.EarthColor,
				},
				[9] = {
					type = "colorpicker",
					name = "Magic Damage",
					getFunc = function() return GetColor("Magic") end,
					setFunc = function(r, g, b) SetColor("Magic", r, g, b) end,
					default = CLog.Default.MagicColor,
				},
				[10] = {
					type = "colorpicker",
					name = "Drowning Damage",
					getFunc = function() return GetColor("Drown") end,
					setFunc = function(r, g, b) SetColor("Drown", r, g, b) end,
					default = CLog.Default.DrownColor,
				},
				[11] = {
					type = "colorpicker",
					name = "Disease Damage",
					getFunc = function() return GetColor("Disease") end,
					setFunc = function(r, g, b) SetColor("Disease", r, g, b) end,
					default = CLog.Default.DiseaseColor,
				},
				[12] = {
					type = "colorpicker",
					name = "Poison Damage",
					getFunc = function() return GetColor("Poison") end,
					setFunc = function(r, g, b) SetColor("Poison", r, g, b) end,
					default = CLog.Default.PoisonColor,
				},
			},
		},
		[23] = {
			type = "checkbox",
			name = "Toggle Buff Colors",
			tooltip = "Toggles the colors of Buffs",
			getFunc = function() return CLog.settings.BuffColor end, 
			setFunc = function(val) CLog.settings.BuffColor = val end,
			default = CLog.Default.BuffColor,
		},
		[24] = {
			type = "submenu",
			name = "Set Buff Colors",
			tooltip = "Expand to set colors for different kinds of buffs",
			controls = {
				[1] = {
					type = "colorpicker",
					name = "Special Effects Buffs",
					getFunc = function() return GetColor("Special") end,
					setFunc = function(r, g, b) SetColor("Special", r, g, b) end,
					default = CLog.Default.SpecialColor,
				},
				[2] = {
					type = "colorpicker",
					name = "Good Buffs",
					getFunc = function() return GetColor("GoodBuff") end,
					setFunc = function(r, g, b) SetColor("GoodBuff", r, g, b) end,
					default = CLog.Default.GoodBuffColor,
				},
				[3] = {
					type = "colorpicker",
					name = "Bad Buffs",
					getFunc = function() return GetColor("BadBuff") end,
					setFunc = function(r, g, b) SetColor("BadBuff", r, g, b) end,
					default = CLog.Default.BadBuffColor,
				},
				[4] = {
					type = "colorpicker",
					name = "Food Buffs",
					getFunc = function() return GetColor("FoodBuff") end,
					setFunc = function(r, g, b) SetColor("FoodBuff", r, g, b) end,
					FoodBuffColor,
				},
			},
		},

		[25] = {
			type = "header",
			name = "|cF0C300Various Tracking",
		},
		[26] = {
			type = "checkbox",
			name = "Show Damage in Log",
			tooltip = "Toggles Damage in Combat Log",
			getFunc = function() return CLog.settings.ShowDamage end,
			setFunc = function(val) CLog.settings.ShowDamage = val end,
			default = CLog.Default.ShowDamage,
		},
		[27] = {
			type = "checkbox",
			name = "Show Healing in Log",
			tooltip = "Toggles Healing in Combat Log",
			getFunc = function() return CLog.settings.ShowHeals end,
			setFunc = function(val) CLog.settings.ShowHeals = val end,
			default = CLog.Default.ShowHeals,
		},
		[28] = {
			type = "checkbox",
			name = "Show Buffs/Debuffs in Log",
			tooltip = "Toggles Buff/Debuff in Combat Log",
			getFunc = function() return CLog.settings.Buffs end,
			setFunc = function(val) CLog.settings.Buffs = val end,
			default = CLog.Default.Buffs,
		},
		[29] = {
			type = "checkbox",
			name = "Show AP Gains",
			tooltip = "Toggles Alliance Points Gains in Combat Log",
			getFunc = function() return CLog.settings.AP end,
			setFunc = function(val) CLog.settings.AP = val end,
			default = CLog.Default.AP,
		},
		[30] = {
			type = "checkbox",
			name = "Show Mag/Stam/Ult Gains",
			tooltip = "Toggles Stamina/Magicka/Ultimate Gains in Combat Log",
			getFunc = function() return CLog.settings.ShowPower end,
			setFunc = function(val) CLog.settings.ShowPower = val end,
			default = CLog.Default.ShowPower,
		},
	}

	--------Skills----------
	for Category, Ntable in pairs(CLog.SkillNames) do
		for k, v in pairs(Ntable) do
			if (CLog.SkillNames[Category][k]) and (CLog.SkillNames[Category][k] ~= "") then
				local skill = {
					type = "checkbox",
					name = "Show |cc8ff3a" .. CLog.SkillNames[Category][k] .. "|r Skill Gains",
					tooltip = "Toggles " .. CLog.SkillNames[Category][k] .. " in Combat Log",
					getFunc = function() return CLog.settings.ShowSkills[Category][k] end,
					setFunc = function(val) CLog.settings.ShowSkills[Category][k] = val end,
					default = true,
				}
				table.insert(optionsTable, skill)
			else
				CLog.settings.ShowSkills[Category][k] = false
			end
		end
	end

	table.insert(optionsTable,
		{
			type = "header",
			name = "|cF0C300Reset Window Positions",
		}
	)
	table.insert(optionsTable, 
		{
			type = "button",
			name = "Reset Main Window Position",
			tooltip = "Click to reset the Main CLS Window to its default position in the upper-left corner of the screen",
			func = function()
				CLog.settings.SumX = 0
				CLog.settings.SumY = 0
				CLog.MainWindow:ClearAnchors()
				CLog.MainWindow:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, CLog.settings.SumX, CLog.settings.SumY)
			end,
		}
	)
	table.insert(optionsTable,
		{
			type = "button",
			name = "Reset DPS Window Position",
			tooltip = "Click to reset the DPS Meter to its default position in the upper-left corner of the screen",
			func = function()
				CLog.settings.SumXDPS = 0
				CLog.settings.SumYDPS = 0
				CLog.DPSWindow:ClearAnchors()
				CLog.DPSWindow:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, CLog.settings.SumXDPS, CLog.settings.SumYDPS)
			end,
		}
	)

	CLog.LAM:RegisterOptionControls("CLS", optionsTable)
end
